#!/usr/bin/env node

const AWS = require('aws-sdk');
const https = require('https');
const fs = require('fs');
const AdmZip = require('adm-zip');
const options = require('node-options');
const colors = require('colors/safe');
const { promised_action, promised_action_no_error } = require('./promises');

async function downloadLambdaCode(functionName, packageFile) {
    var lambda = new AWS.Lambda();
    var functionInfo = await lambda.getFunction({ FunctionName: functionName }).promise();

    console.log(`Downloading source code`);
    var response = await promised_action_no_error((callback) => https.get(functionInfo.Code.Location, callback));

    if (response.statusCode == 200) {
        var writableStream = fs.createWriteStream(packageFile);

        response.pipe(writableStream);

        await promised_action_no_error((callback) => {
            response.on('end', callback);
        });

        writableStream.close();

        await promised_action_no_error((callback) => {
            writableStream.on('close', callback);
        });
    }
}

async function unzipLambdaCode(packageFile, extractToPath) {
    console.log(`Unzipping: ${packageFile}`);

    var zip = new AdmZip(packageFile);
    zip.extractAllTo(extractToPath, true);
}

async function uploadLambdaCode(functionName, packageFile, s3Bucket, revisionId) {
    const s3 = new AWS.S3();
    const lambda = new AWS.Lambda();
    const s3Key = `${functionName}.zip`;

    var readableStream = fs.createReadStream(packageFile);

    await s3.putObject({
        Bucket: s3Bucket,
        Key: s3Key,
        Body: readableStream
    }).promise();

    var updateParams = {
        FunctionName: functionName,
        S3Bucket: s3Bucket,
        S3Key: s3Key,
        Publish: true
    };

    if (revisionId !== null) {
        updateParams.RevisionId = revisionId;
    }

    await lambda.updateFunctionCode(updateParams).promise();
}

async function zipLambdaCode(packageFile, sourceRoot) {
    var zip = new AdmZip();
    zip.addLocalFolder(sourceRoot);
    zip.writeZip(packageFile);
}

async function updateLambda(functionName, packageZipFile, sourceRoot, s3Bucket, event, fileName, revisionId) {
    process.stdout.write(colors.green(`Updating...`));
    await zipLambdaCode(packageZipFile, sourceRoot);
    await uploadLambdaCode(functionName, packageZipFile, s3Bucket, revisionId)

    process.stdout.clearLine();
    process.stdout.cursorTo(0);
}

async function execute() {
    var appOptions = {
        "function-name": "",
        "s3-bucket": "",
        "aws-region": "us-east-1",
        "aws-profile": "",
        "change-delay": 5,
        "revision-id": null
    };

    options.parse(process.argv.slice(2), appOptions);

    const region = appOptions['aws-region'];
    const profile = appOptions['aws-profile'];
    const functionName = appOptions['function-name'];
    const s3Bucket = appOptions['s3-bucket'];
    const revisionId = appOptions['revision-id'];
    const changeDelay = parseInt(appOptions['change-delay']) * 1000;

    if (functionName == "" || s3Bucket == "") {
        console.log('USAGE: lambda-watch --function-name=<name> --s3-bucket=<bucket> [--aws-region=<region>] [--aws-profile=<name>] [--change-delay=5] [--revision-id=$LATEST]');
        return;
    }

    const packageZipFile = `${functionName}.zip`;
    const sourceRoot = `${functionName}/`;

    if (profile != '') {
        AWS.config.credentials = new AWS.SharedIniFileCredentials({ profile });
        console.log(`Using AWS Profile: ${profile}`);
    }

    AWS.config.update({ region });

    await downloadLambdaCode(functionName, packageZipFile);
    await unzipLambdaCode(packageZipFile, sourceRoot);

    console.log(`Watching source folder: ${sourceRoot}`);
    console.log();
    var timerId = null;
    fs.watch(sourceRoot, { recursive: true }, (event, fileName) => {
        if (timerId === null) {
            process.stdout.write(colors.green(`Update Pending...`));
        }

        clearTimeout(timerId);
        timerId = setTimeout(() => {
            process.stdout.clearLine();
            process.stdout.cursorTo(0);

            updateLambda(functionName, packageZipFile, sourceRoot, s3Bucket, event, fileName, revisionId);
            timerId = null;
        }, changeDelay);
    });
}

execute()
    .then(() => {})
    .catch((err) => {
        console.error(`Error: ${err} / ${err.stack}`);
    });
